@@ -1,4 +1,3 @@
-/* $Xorg: omGeneric.c,v 1.6 2000/08/17 19:45:21 cpqbld Exp $ */
 /*  #define FONTDEBUG */
 /*
  * Copyright 1992, 1993 by TOSHIBA Corp.
@@ -32,7 +31,6 @@
  * Modifier:  Takanori Tateno   FUJITSU LIMITED
  *
  */
-/* $XFree86: xc/lib/X11/omGeneric.c,v 3.27 2003/05/27 22:26:27 tsi Exp $ */
 
 /*
  * Fixed the algorithms in parse_fontname() and parse_fontdata()
@@ -41,7 +39,7 @@
  * so that you can figure out what in the heck we're doing. We
  * realize this is a departure from the norm, but hey, we're
  * rebels! :-) :-)
- * 
+ *
  * Modifiers: Jeff Walls, Paul Anderson: HEWLETT-PACKARD
  */
 /*
@@ -67,32 +65,6 @@
 #define	CHARSET_ENCODING_FIELD	14
 #define XLFD_MAX_LEN		255
 
-#if 0
-extern int _XmbDefaultTextEscapement(), _XwcDefaultTextEscapement(),
-	   _Xutf8DefaultTextEscapement();
-extern int _XmbDefaultTextExtents(), _XwcDefaultTextExtents(),
-	   _Xutf8DefaultTextExtents();
-extern Status _XmbDefaultTextPerCharExtents(), _XwcDefaultTextPerCharExtents(),
-	      _Xutf8DefaultTextPerCharExtents();
-extern int _XmbDefaultDrawString(), _XwcDefaultDrawString(),
-	   _Xutf8DefaultDrawString();
-extern void _XmbDefaultDrawImageString(), _XwcDefaultDrawImageString(),
-	    _Xutf8DefaultDrawImageString();
-
-extern int _XmbGenericTextEscapement(), _XwcGenericTextEscapement(),
-	   _Xutf8GenericTextEscapement();
-extern int _XmbGenericTextExtents(), _XwcGenericTextExtents(),
-	   _Xutf8GenericTextExtents();
-extern Status _XmbGenericTextPerCharExtents(), _XwcGenericTextPerCharExtents(),
-	      _Xutf8GenericTextPerCharExtents();
-extern int _XmbGenericDrawString(), _XwcGenericDrawString(),
-	   _Xutf8GenericDrawString();
-extern void _XmbGenericDrawImageString(), _XwcGenericDrawImageString(),
-	    _Xutf8GenericDrawImageString();
-
-extern void _XlcDbg_printValue (const char *str, char **value, int num);
-#endif
-
 /* For VW/UDC start */
 
 static FontData
@@ -103,11 +75,10 @@ init_fontdata(
     FontData	fd;
     int		i;
 
-    fd = (FontData)Xmalloc(sizeof(FontDataRec) * font_data_count);
+    fd = Xcalloc(font_data_count, sizeof(FontDataRec));
     if(fd == (FontData) NULL)
 	return False;
 
-    memset(fd, 0x00, sizeof(FontData) * font_data_count);
     for(i = 0 ; i < font_data_count ; i++)
 	fd[i] = font_data[i];
 
@@ -128,11 +99,10 @@ init_vrotate(
     if(type == VROTATE_NONE)
 	return (VRotate)NULL;
 
-    vrotate = (VRotate)Xmalloc(sizeof(VRotateRec) * font_data_count);
+    vrotate = Xcalloc(font_data_count, sizeof(VRotateRec));
     if(vrotate == (VRotate) NULL)
 	return False;
 
-    memset(vrotate, 0x00, sizeof(VRotateRec) * font_data_count);
     for(i = 0 ; i < font_data_count ; i++) {
 	vrotate[i].charset_name = font_data[i].name;
 	vrotate[i].side = font_data[i].side;
@@ -157,10 +127,9 @@ init_fontset(
     count = XOM_GENERIC(oc->core.om)->data_num;
     data = XOM_GENERIC(oc->core.om)->data;
 
-    font_set = (FontSet) Xmalloc(sizeof(FontSetRec) * count);
+    font_set = Xcalloc(count, sizeof(FontSetRec));
     if (font_set == NULL)
 	return False;
-    memset((char *) font_set, 0x00, sizeof(FontSetRec) * count);
 
     gen = XOC_GENERIC(oc);
     gen->font_set_num = count;
@@ -198,16 +167,12 @@ init_fontset(
     return True;
 
 err:
-    if(font_set->font_data)
-	Xfree(font_set->font_data);
-    if(font_set->substitute)
-	Xfree(font_set->substitute);
-    if(font_set->vmap)
-	Xfree(font_set->vmap);
-    if(font_set->vrotate)
-	Xfree(font_set->vrotate);
-    if(font_set)
-	Xfree(font_set);
+
+    Xfree(font_set->font_data);
+    Xfree(font_set->substitute);
+    Xfree(font_set->vmap);
+    Xfree(font_set->vrotate);
+    Xfree(font_set);
     gen->font_set = (FontSet) NULL;
     gen->font_set_num = 0;
     return False;
@@ -223,7 +188,7 @@ get_prop_name(
     unsigned long fp;
 
     if (XGetFontProperty(fs, XA_FONT, &fp))
-	return XGetAtomName(dpy, fp); 
+	return XGetAtomName(dpy, fp);
 
     return (char *) NULL;
 }
@@ -261,7 +226,7 @@ load_fontset_data(
     if(font_set->font_name == (char *)NULL) return False ;
 
    /* If font_set->font is not NULL, it contains the *best*
-    * match font for this FontSet.  
+    * match font for this FontSet.
     * -- jjw/pma (HP)
     */
     if(font_set->font == NULL) {
@@ -338,7 +303,7 @@ load_font_info(
 					 &font_set->info);
 	    if (font_set->info == NULL)
 		return False;
-	    
+
 	    XFreeFontNames(fn_list);
 	}
     }
@@ -412,11 +377,10 @@ set_fontset_extents(
 		}
 	    }
 
-	    if(font_set->vrotate_num > 0) {
+	    if(font_set->vrotate_num > 0 && font_set->vrotate != NULL) {
 		font_data = (FontData) font_set->vrotate;
 		font_data_count = font_set->vrotate_num;
 		for( ; font_data_count-- ; font_data++) {
-		    if(font_data != NULL)
 		    if(font_data->font != NULL) {
 			check_fontset_extents(&overall, &logical_ascent,
 					      &logical_descent,
@@ -466,15 +430,15 @@ init_core_part(
     if (count == 0)
         return False;
 
-    font_struct_list = (XFontStruct **) Xmalloc(sizeof(XFontStruct *) * count);
+    font_struct_list = Xmalloc(sizeof(XFontStruct *) * count);
     if (font_struct_list == NULL)
 	return False;
 
-    font_name_list = (char **) Xmalloc(sizeof(char *) * count);
+    font_name_list = Xmalloc(sizeof(char *) * count);
     if (font_name_list == NULL)
 	goto err;
 
-    font_name_buf = (char *) Xmalloc(length);
+    font_name_buf = Xmalloc(length);
     if (font_name_buf == NULL)
 	goto err;
 
@@ -507,8 +471,8 @@ init_core_part(
     return True;
 
 err:
-    if (font_name_list)
-	Xfree(font_name_list);
+
+    Xfree(font_name_list);
     Xfree(font_struct_list);
 
     return False;
@@ -526,10 +490,8 @@ get_font_name(
     if (list == NULL)
 	return NULL;
 
-    name = (char *) Xmalloc(strlen(*list) + 1);
-    if (name)
-	strcpy(name, *list);
-    
+    name = strdup(*list);
+
     XFreeFontNames(list);
 
     return name;
@@ -552,10 +514,9 @@ get_rotate_fontname(
        || len > XLFD_MAX_LEN)
 	return NULL;
 
-    pattern = (char *)Xmalloc(len + 1);
+    pattern = strdup(font_name);
     if(!pattern)
 	return NULL;
-    strcpy(pattern, font_name);
 
     memset(fields, 0, sizeof(char *) * 14);
     ptr = pattern;
@@ -568,14 +529,17 @@ get_rotate_fontname(
     for(field_num = 0 ; field_num < CHARSET_ENCODING_FIELD && ptr && *ptr ;
 			ptr++, field_num++) {
 	fields[field_num] = ptr;
-	
+
 	if((ptr = strchr(ptr, '-'))) {
 	    *ptr = '\0';
+	} else {
+	    field_num++;	/* Count last field */
+	    break;
 	}
     }
 
     if(field_num < CHARSET_ENCODING_FIELD)
-	return NULL;
+	goto free_pattern;
 
     /* Pixel Size field : fields[6] */
     for(ptr = fields[PIXEL_SIZE_FIELD - 1] ; ptr && *ptr; ptr++) {
@@ -584,13 +548,12 @@ get_rotate_fontname(
 	        strcpy(pattern, font_name);
 		return(pattern);
 	    }
-	    if(pattern)
-		Xfree(pattern);
-	    return NULL;
+	    goto free_pattern;
 	}
     }
     pixel_size = atoi(fields[PIXEL_SIZE_FIELD - 1]);
-    sprintf(str_pixel, "[ 0 ~%d %d 0 ]", pixel_size, pixel_size);
+    snprintf(str_pixel, sizeof(str_pixel),
+	     "[ 0 ~%d %d 0 ]", pixel_size, pixel_size);
     fields[6] = str_pixel;
 
     /* Point Size field : fields[7] */
@@ -604,22 +567,23 @@ get_rotate_fontname(
     }
 
     /* Max XLFD length is 255 */
-    if (len > XLFD_MAX_LEN) 
-	return NULL;
+    if (len > XLFD_MAX_LEN)
+	goto free_pattern;
 
-    rotate_font_ptr = (char *)Xmalloc(len + 1);
+    rotate_font_ptr = Xmalloc(len + 1);
     if(!rotate_font_ptr)
-	return NULL;
+	goto free_pattern;
 
     rotate_font_ptr[0] = '\0';
 
     for(field_num = 0 ; field_num < CHARSET_ENCODING_FIELD &&
 			fields[field_num] ; field_num++) {
-	sprintf(rotate_font_ptr, "%s-%s", rotate_font_ptr, fields[field_num]);
+	strcat(rotate_font_ptr, "-");
+	strcat(rotate_font_ptr, fields[field_num]);
     }
 
-    if(pattern)
-	Xfree(pattern);
+free_pattern:
+    Xfree(pattern);
 
     return rotate_font_ptr;
 }
@@ -638,43 +602,13 @@ is_match_charset(
     length = strlen(font_data->name);
     if (length > name_len)
 	return False;
-	
+
     if (_XlcCompareISOLatin1(last - length, font_data->name) == 0)
 	return True;
 
     return False;
 }
 
-#if 0
-static char *
-get_font_name_from_list(
-    XOC oc,
-    char *pattern,
-    FontData    font_data)
-{
-    char **list, *name = (char *)NULL, *fname;
-    int count = 0, i;
-
-    list = XListFonts(oc->core.om->core.display, pattern, MAXFONTS, &count);
-    if (list == NULL)
-	return NULL;
-
-    for (i = 0; i < count; i++) {
-        fname = list[i];
-        if(is_match_charset(font_data, fname) == True) {
-             name = (char *) Xmalloc(strlen(fname) + 1);
-             if (name)
-	         strcpy(name, fname);
-             break;
-        }
-    }
-    
-    XFreeFontNames(list);
-
-    return name;
-}
-#endif
-
 static int
 parse_all_name(
     XOC		oc,
@@ -686,11 +620,10 @@ parse_all_name(
     if(is_match_charset(font_data, pattern) != True)
  	return False;
 
-    font_data->xlfd_name = (char *)Xmalloc(strlen(pattern)+1);
+    font_data->xlfd_name = strdup(pattern);
     if(font_data->xlfd_name == NULL)
 	return (-1);
 
-    strcpy(font_data->xlfd_name, pattern);
     return True;
 #else  /* OLDCODE */
     Display *dpy = oc->core.om->core.display;
@@ -724,11 +657,10 @@ parse_all_name(
         }
     }
 
-    font_data->xlfd_name = (char *)Xmalloc(strlen(pattern)+1);
+    font_data->xlfd_name = strdup(pattern);
     if(font_data->xlfd_name == NULL)
 	return (-1);
 
-    strcpy(font_data->xlfd_name, pattern);
     return True;
 #endif /* OLDCODE */
 }
@@ -758,7 +690,7 @@ parse_omit_name(
 
     if (length > XLFD_MAX_LEN)
 	return -1;
-    
+
     strcpy(buf, pattern);
     last = buf + length - 1;
 
@@ -771,12 +703,12 @@ parse_omit_name(
     switch (num_fields) {
     case 12:
 	/* This is the best way to have specifed the fontset.  In this
-	 * case, there is no original encoding. E.g., 
+	 * case, there is no original encoding. E.g.,
          *       -*-*-*-*-*-*-14-*-*-*-*-*
 	 * To this, we'll append a dash:
          *       -*-*-*-*-*-*-14-*-*-*-*-*-
 	 * then append the encoding to get:
-         *       -*-*-*-*-*-*-14-*-*-*-*-*-JISX0208.1990-0 
+         *       -*-*-*-*-*-*-14-*-*-*-*-*-JISX0208.1990-0
 	 */
 	/*
 	 * Take care of:
@@ -789,7 +721,7 @@ parse_omit_name(
 	break;
     case 13:
 	/* Got the charset, not the encoding, zap the charset  In this
-	 * case, there is no original encoding, but there is a charset. E.g., 
+	 * case, there is no original encoding, but there is a charset. E.g.,
          *       -*-*-*-*-*-*-14-*-*-*-*-*-jisx0212.1990
 	 * To this, we remove the charset:
          *       -*-*-*-*-*-*-14-*-*-*-*-*-
@@ -802,11 +734,11 @@ parse_omit_name(
     case 14:
 	/* Both the charset and the encoding are specified.  Get rid
 	 * of them so that we can append the new charset encoding.  E.g.,
-         *       -*-*-*-*-*-*-14-*-*-*-*-*-jisx0212.1990-0 
+         *       -*-*-*-*-*-*-14-*-*-*-*-*-jisx0212.1990-0
 	 * To this, we'll remove the encoding and charset to get:
          *       -*-*-*-*-*-*-14-*-*-*-*-*-
 	 * then append the new encoding to get:
-         *       -*-*-*-*-*-*-14-*-*-*-*-*-JISX0208.1990-0 
+         *       -*-*-*-*-*-*-14-*-*-*-*-*-JISX0208.1990-0
 	 */
 	last = strrchr (buf, '-');
 	*last = '\0';
@@ -819,17 +751,17 @@ parse_omit_name(
 	break;
     }
 
-   /* At this point, "last" is pointing to the last "-" in the 
+   /* At this point, "last" is pointing to the last "-" in the
     * xlfd, and all xlfd's at this point take a form similar to:
     *       -*-*-*-*-*-*-14-*-*-*-*-*-
     * (i.e., no encoding).
     * After the strcpy, we'll end up with something similar to:
-    *       -*-*-*-*-*-*-14-*-*-*-*-*-JISX0208.1990-0 
+    *       -*-*-*-*-*-*-14-*-*-*-*-*-JISX0208.1990-0
     *
-    * If the modified font is found in the current FontSet, 
+    * If the modified font is found in the current FontSet,
     * we've found a match.
     */
-    
+
     last++;
 
     if ((last - buf) + strlen(font_data->name) > XLFD_MAX_LEN)
@@ -840,18 +772,18 @@ parse_omit_name(
 	return True;
 
     /* This may mot be needed anymore as XListFonts() takes care of this */
-    while (num_fields < 12) {
+    if (num_fields < 12) {
 	if ((last - buf) > (XLFD_MAX_LEN - 2))
 	    return -1;
 	*last = '*';
 	*(last + 1) = '-';
 	strcpy(last + 2, font_data->name);
-	num_fields++; 
+	num_fields++;
 	last+=2;
 	if ((font_data->xlfd_name = get_font_name(oc, buf)) != NULL)
-	    return True;	    
+	    return True;
     }
-    
+
 
     return False;
 }
@@ -884,7 +816,7 @@ parse_fontdata(
     if(font_data == NULL || font_data_count <= 0) {
 	return False;
     }
-    
+
     /* Loop through each font encoding defined in the "font_data" FontSet. */
     for ( ; font_data_count-- > 0; font_data++) {
 	Bool	is_found = False;
@@ -904,7 +836,7 @@ parse_fontdata(
 		fprintf(stderr,"Font pattern: %s %s\n",
 		pattern,font_data->name);
 #endif
-	    
+
 	    /*
 	     * If the current font is fully specified (i.e., the
 	     * xlfd contains no wildcards) and the font exists on
@@ -925,16 +857,16 @@ parse_fontdata(
 	        if (ret == False) continue;
                /*
 		* Since there was an exact match of a fully-specified font
-		* or a font alias, we can return now since the desired font 
+		* or a font alias, we can return now since the desired font
 		* was found for the current font encoding for this FontSet.
 		*
 		* Previous implementations of this algorithm would
-		* not return here. Instead, they continued searching 
-		* through the font encodings for this FontSet. The side-effect 
-		* of that behavior is you may return a "substitute" match 
-		* instead of an "exact" match.  We believe there should be a 
+		* not return here. Instead, they continued searching
+		* through the font encodings for this FontSet. The side-effect
+		* of that behavior is you may return a "substitute" match
+		* instead of an "exact" match.  We believe there should be a
 		* preference on exact matches.  Therefore, as soon as we
-		* find one, we bail. 
+		* find one, we bail.
 		*
 		* Also, previous implementations seemed to think it was
 		* important to find either a primary or substitute font
@@ -942,17 +874,14 @@ parse_fontdata(
 		* acceptable font.  We don't believe this is necessary.
 		* All the client cares about is finding a reasonable font
 		* for what was passed in.  If we find an exact match,
-		* there's no reason to look any further.  
+		* there's no reason to look any further.
 		*
 		* -- jjw/pma (HP)
 		*/
 		if (font_data_return) {
-		    font_data_return->xlfd_name = (char *)Xmalloc
-			(strlen(font_data->xlfd_name) + 1);
+		    font_data_return->xlfd_name = strdup(font_data->xlfd_name);
 		    if (!font_data_return->xlfd_name) return -1;
 
-		    strcpy (font_data_return->xlfd_name, font_data->xlfd_name);
-
 		    font_data_return->side      = font_data->side;
 		}
 #ifdef FONTDEBUG
@@ -973,7 +902,7 @@ parse_fontdata(
 	    if (ret == False) continue;
 
            /*
-	    * A font which matched the wild-carded specification was found. 
+	    * A font which matched the wild-carded specification was found.
 	    * Only update the return data if a font has not yet been found.
 	    * This maintains the convention that FontSets listed higher in
 	    * a CodeSet in the Locale Database have higher priority than
@@ -990,18 +919,16 @@ parse_fontdata(
 	    *
 	    * a font found in the JISX0208.1990-0 FontSet will have a
 	    * higher priority than a font found in the JISX0208.1983-0
-	    * FontSet.  
+	    * FontSet.
 	    */
 	    if (font_data_return && font_data_return->xlfd_name == NULL) {
 
 #ifdef FONTDEBUG
 		fprintf(stderr,"XLFD name: %s\n",font_data->xlfd_name);
 #endif
-		font_data_return->xlfd_name = (char *)Xmalloc
-			(strlen(font_data->xlfd_name) + 1);
+		font_data_return->xlfd_name = strdup(font_data->xlfd_name);
                 if (!font_data_return->xlfd_name) return -1;
 
-	        strcpy (font_data_return->xlfd_name, font_data->xlfd_name);
 	        font_data_return->side      = font_data->side;
 	    }
 
@@ -1017,7 +944,7 @@ parse_fontdata(
 		 /*
 		  * Did not find a font for the current FontSet.  Check the
 		  * FontSet's "substitute" font for a match.  If we find a
-		  * match, we'll keep searching in hopes of finding an exact 
+		  * match, we'll keep searching in hopes of finding an exact
 		  * match later down the FontSet list.
 		  *
 		  * when we return and we have found a font font_data_return
@@ -1025,8 +952,8 @@ parse_fontdata(
 		  * is a C_PRIMARY or a C_SUBSTITUTE font
 		  */
 		  ret = parse_fontdata(oc, font_set, font_set->substitute,
-				       font_set->substitute_num, name_list, 
-				       name_list_count, C_SUBSTITUTE, 
+				       font_set->substitute_num, name_list,
+				       name_list_count, C_SUBSTITUTE,
 				       font_data_return);
                   if (ret == -1)    return -1;
 		  if (ret == False) continue;
@@ -1110,12 +1037,11 @@ parse_vw(
 	    Xfree(vrotate);
 
 	    if(sub_num > 0) {
-		vrotate = font_set->vrotate = (VRotate)Xmalloc
-						(sizeof(VRotateRec) * sub_num);
+		vrotate = font_set->vrotate = Xcalloc(sub_num,
+                                                      sizeof(VRotateRec));
 		if(font_set->vrotate == (VRotate)NULL)
 		    return (-1);
-		memset(font_set->vrotate, 0x00, sizeof(VRotateRec) * sub_num);
-		
+
 		for(i = 0 ; i < sub_num ; i++) {
 		    vrotate[i].charset_name = font_set->substitute[i].name;
 		    vrotate[i].side = font_set->substitute[i].side;
@@ -1168,15 +1094,15 @@ parse_fontname(
 	if(font_set->font_data_count > 0) {
 
            /*
-	    * If there are a non-zero number of FontSets defined 
+	    * If there are a non-zero number of FontSets defined
 	    * for this CharSet.
             * Try to find a font for this CharSet.  If we find an
 	    * acceptable font, we save the information for return
 	    * to the client.  If we do not find an acceptable font,
-	    * a "missing_charset" will be reported to the client 
+	    * a "missing_charset" will be reported to the client
 	    * for this CharSet.
 	    */
-	    font_data_return. xlfd_name = NULL;
+	    font_data_return.xlfd_name  = NULL;
 	    font_data_return.side       = XlcUnknown;
 
 	    ret = parse_fontdata(oc, font_set, font_set->font_data,
@@ -1193,11 +1119,10 @@ parse_fontname(
 		 * be matched. It returns the required information in
 		 * font_data_return.
 		 */
-		font_set->font_name = (char *)Xmalloc
-			(strlen(font_data_return.xlfd_name) + 1);
+		font_set->font_name = strdup(font_data_return.xlfd_name);
 		if(font_set->font_name == (char *) NULL)
 		    goto err;
-		strcpy(font_set->font_name, font_data_return.xlfd_name);
+
 		font_set->side = font_data_return.side;
 
                 Xfree (font_data_return.xlfd_name);
@@ -1210,7 +1135,7 @@ parse_fontname(
 
 	} else if(font_set->substitute_num > 0) {
            /*
-	    * If there are no FontSets defined for this 
+	    * If there are no FontSets defined for this
 	    * CharSet.  We can only find "substitute" fonts.
 	    */
 	    ret = parse_fontdata(oc, font_set, font_set->substitute,
@@ -1224,11 +1149,10 @@ parse_fontname(
 			break;
 		    }
 		}
-		font_set->font_name = (char *)Xmalloc
-		    	(strlen(font_set->substitute[i].xlfd_name) + 1);
+		font_set->font_name = strdup(font_set->substitute[i].xlfd_name);
 		if(font_set->font_name == (char *) NULL)
 		    goto err;
-		strcpy(font_set->font_name,font_set->substitute[i].xlfd_name);
+
 		font_set->side = font_set->substitute[i].side;
 		if(parse_vw(oc, font_set, name_list, count) == -1)
 		    goto err;
@@ -1238,11 +1162,10 @@ parse_fontname(
 	}
     }
 
-    base_name = (char *) Xmalloc(strlen(oc->core.base_name_list) + 1);
+    base_name = strdup(oc->core.base_name_list);
     if (base_name == NULL)
 	goto err;
 
-    strcpy(base_name, oc->core.base_name_list);
     oc->core.base_name_list = base_name;
 
     XFreeStringList(name_list);
@@ -1277,14 +1200,14 @@ set_missing_list(
 	if (font_set->info || font_set->font) {
 	    continue;
 	}
-	
+
 	/* Change 1996.01.23 start */
 	if(font_set->font_data_count <= 0 ||
 	   font_set->font_data == (FontData)NULL) {
 	    if(font_set->substitute_num <= 0 ||
 	       font_set->substitute == (FontData)NULL) {
 		if(font_set->charset_list != NULL){
-		 length += 
+		 length +=
 		  strlen(font_set->charset_list[0]->encoding_name) + 1;
 		} else {
 		  length += 1;
@@ -1303,12 +1226,12 @@ set_missing_list(
 	return True;
     }
 
-    charset_list = (char **) Xmalloc(sizeof(char *) * count);
+    charset_list = Xmalloc(sizeof(char *) * count);
     if (charset_list == NULL) {
 	return False;
     }
 
-    charset_buf = (char *) Xmalloc(length);
+    charset_buf = Xmalloc(length);
     if (charset_buf == NULL) {
 	Xfree(charset_list);
 	return False;
@@ -1346,7 +1269,7 @@ set_missing_list(
 	/* Change 1996.01.23 end */
 	*charset_list++ = charset_buf;
 	charset_buf += strlen(charset_buf) + 1;
-    } 
+    }
 
     if(result == 0) {
 	return(False);
@@ -1408,7 +1331,7 @@ free_fontdataOC(
 		XFreeFontInfo(NULL, font_data->font, 1);/* Add 1996.01.23 */
 	    font_data->font = NULL;
 	}
-/* 
+/*
  * font_data->name and font_data->scopes belong to the OM not OC.
  * To save space this data is shared between OM and OC. We are
  * not allowed to free it here.
@@ -1496,31 +1419,22 @@ destroy_oc(
 	_XlcCloseConverter(gen->utf8_to_cs);
 
 /* For VW/UDC start */ /* Change 1996.01.8 */
-    destroy_fontdata(gen,dpy); 
+    destroy_fontdata(gen,dpy);
 /*
 */
 /* For VW/UDC end */
 
-    if (oc->core.base_name_list)
-	Xfree(oc->core.base_name_list);
-
-    if (oc->core.font_info.font_name_list)
-	XFreeStringList(oc->core.font_info.font_name_list);
-
-    if (oc->core.font_info.font_struct_list) {
-	Xfree(oc->core.font_info.font_struct_list);
-    }
-
-    if (oc->core.missing_list.charset_list)
-	XFreeStringList(oc->core.missing_list.charset_list);
+    Xfree(oc->core.base_name_list);
+    XFreeStringList(oc->core.font_info.font_name_list);
+    Xfree(oc->core.font_info.font_struct_list);
+    XFreeStringList(oc->core.missing_list.charset_list);
 
 #ifdef notdef
-    if (oc->core.res_name)
-	Xfree(oc->core.res_name);
-    if (oc->core.res_class)
-	Xfree(oc->core.res_class);
+
+    Xfree(oc->core.res_name);
+    Xfree(oc->core.res_class);
 #endif
-    
+
     Xfree(oc);
 }
 
@@ -1616,7 +1530,7 @@ static XOCMethodsRec oc_generic_methods = {
 };
 
 typedef struct _XOCMethodsListRec {
-    char *name;
+    const char *name;
     XOCMethods methods;
 } XOCMethodsListRec, *XOCMethodsList;
 
@@ -1655,16 +1569,15 @@ create_oc(
     XOCMethodsList methods_list = oc_methods_list;
     int count;
 
-    oc = (XOC) Xmalloc(sizeof(XOCGenericRec));
+    oc = Xcalloc(1, sizeof(XOCGenericRec));
     if (oc == NULL)
 	return (XOC) NULL;
-    bzero((char *) oc, sizeof(XOCGenericRec));
-    
+
     oc->core.om = om;
 
     if (oc_resources[0].xrm_name == NULLQUARK)
 	_XlcCompileResourceList(oc_resources, XlcNumber(oc_resources));
-    
+
     if (_XlcSetValues((XPointer) oc, oc_resources, XlcNumber(oc_resources),
 		      args, num_args, XlcCreateMask | XlcDefaultMask))
 	goto err;
@@ -1704,15 +1617,14 @@ free_fontdataOM(
     FontData	font_data,
     int		font_data_count)
 {
+    if (!font_data)
+	return;
+
     for( ; font_data_count-- ; font_data++) {
-	if(font_data->name){
 	    Xfree(font_data->name);
 	    font_data->name = NULL;
-	}
-	if(font_data->scopes){
 	    Xfree(font_data->scopes);
 	    font_data->scopes = NULL;
-	}
     }
 }
 
@@ -1726,51 +1638,41 @@ close_om(
 
     if ((data = gen->data)) {
 	for (count = gen->data_num; count-- > 0; data++) {
-	    if (data->charset_list){
 		Xfree(data->charset_list);
 		data->charset_list = NULL;
-	    }
+	
 	    /* free font_data for om */
-	    if (data->font_data) {
 		free_fontdataOM(data->font_data,data->font_data_count);
 		Xfree(data->font_data);
 		data->font_data = NULL;
-	    }
+
 	    /* free substitute for om */
-	    if (data->substitute) {
 		free_fontdataOM(data->substitute,data->substitute_num);
 		Xfree(data->substitute);
 		data->substitute = NULL;
-	    }
+
 	    /* free vmap for om */
-	    if (data->vmap) {
 		free_fontdataOM(data->vmap,data->vmap_num);
 		Xfree(data->vmap);
 		data->vmap = NULL;
-	    }
+
 	    /* free vrotate for om */
-	    if (data->vrotate) {
 		Xfree(data->vrotate);
 		data->vrotate = NULL;
-	    }
 	}
 	Xfree(gen->data);
 	gen->data = NULL;
     }
 
-    if (gen->object_name){
 	Xfree(gen->object_name);
 	gen->object_name = NULL;
-    }
 
-    if (om->core.res_name){
 	Xfree(om->core.res_name);
 	om->core.res_name = NULL;
-    }
-    if (om->core.res_class){
+
 	Xfree(om->core.res_class);
 	om->core.res_class = NULL;
-    }
+
     if (om->core.required_charset.charset_list &&
 	om->core.required_charset.charset_count > 0){
 	XFreeStringList(om->core.required_charset.charset_list);
@@ -1779,10 +1681,9 @@ close_om(
 	Xfree((char*)om->core.required_charset.charset_list);
 	om->core.required_charset.charset_list = NULL;
     }
-    if (om->core.orientation_list.orientation){
+
 	Xfree(om->core.orientation_list.orientation);
 	om->core.orientation_list.orientation = NULL;
-    }
 
     Xfree(om);
 
@@ -1843,31 +1744,28 @@ create_om(
 {
     XOM om;
 
-    om = (XOM) Xmalloc(sizeof(XOMGenericRec));
+    om = Xcalloc(1, sizeof(XOMGenericRec));
     if (om == NULL)
 	return (XOM) NULL;
-    bzero((char *) om, sizeof(XOMGenericRec));
-    
+
     om->methods = &methods;
     om->core.lcd = lcd;
     om->core.display = dpy;
     om->core.rdb = rdb;
     if (res_name) {
-	om->core.res_name = (char *) Xmalloc(strlen(res_name) + 1);
+	om->core.res_name = strdup(res_name);
 	if (om->core.res_name == NULL)
 	    goto err;
-	strcpy(om->core.res_name, res_name);
     }
     if (res_class) {
-	om->core.res_class = (char *) Xmalloc(strlen(res_class) + 1);
+	om->core.res_class = strdup(res_class);
 	if (om->core.res_class == NULL)
 	    goto err;
-	strcpy(om->core.res_class, res_class);
     }
 
     if (om_resources[0].xrm_name == NULLQUARK)
 	_XlcCompileResourceList(om_resources, XlcNumber(om_resources));
-    
+
     om->core.resources = om_resources;
     om->core.num_resources = XlcNumber(om_resources);
 
@@ -1888,9 +1786,9 @@ add_data(
     int num;
 
     if ((num = gen->data_num))
-        new = (OMData) Xrealloc(gen->data, (num + 1) * sizeof(OMDataRec));
+        new = Xrealloc(gen->data, (num + 1) * sizeof(OMDataRec));
     else
-        new = (OMData) Xmalloc(sizeof(OMDataRec));
+        new = Xmalloc(sizeof(OMDataRec));
 
     if (new == NULL)
         return NULL;
@@ -1913,14 +1811,13 @@ read_EncodingInfo(
 {
     FontData font_data,ret;
     char *buf, *bufptr,*scp;
-    int len;
-    font_data = (FontData) Xmalloc(sizeof(FontDataRec) * count);
+    int len, i;
+    font_data = Xcalloc(count, sizeof(FontDataRec));
     if (font_data == NULL)
         return NULL;
-    bzero((char *) font_data, sizeof(FontDataRec) * count);
 
     ret = font_data;
-    for ( ; count-- > 0; font_data++) {
+    for (i = 0; i < count; i++, font_data++) {
 /*
         strcpy(buf, *value++);
 */
@@ -1930,9 +1827,12 @@ read_EncodingInfo(
             bufptr++ ;
 	} else
             len = strlen(buf);
-        font_data->name = (char *) Xmalloc(len + 1);
-        if (font_data->name == NULL)
+        font_data->name = Xmalloc(len + 1);
+        if (font_data->name == NULL) {
+            free_fontdataOM(ret, i + 1);
+            Xfree(ret);
             return NULL;
+	}
         strncpy(font_data->name, buf,len);
 	font_data->name[len] = 0;
         if (bufptr && _XlcCompareISOLatin1(bufptr, "GL") == 0)
@@ -1979,7 +1879,7 @@ static void read_vw(
     char **value, buf[BUFSIZ];
     int count;
 
-    sprintf(buf, "fs%d.font.vertical_map", num);
+    snprintf(buf, sizeof(buf), "fs%d.font.vertical_map", num);
     _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
     if (count > 0){
         _XlcDbg_printValue(buf,value,count);
@@ -1987,7 +1887,7 @@ static void read_vw(
         font_set->vmap = read_EncodingInfo(count,value);
     }
 
-    sprintf(buf, "fs%d.font.vertical_rotate", num);
+    snprintf(buf, sizeof(buf), "fs%d.font.vertical_rotate", num);
     _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
     if (count > 0){
         _XlcDbg_printValue(buf,value,count);
@@ -2016,19 +1916,18 @@ init_om(
 
     _XlcGetResource(lcd, "XLC_FONTSET", "object_name", &value, &count);
     if (count > 0) {
-	gen->object_name = (char *) Xmalloc(strlen(*value) + 1);
+	gen->object_name = strdup(*value);
 	if (gen->object_name == NULL)
 	    return False;
-	strcpy(gen->object_name, *value);
     }
 
     for (num = 0; ; num++) {
 
-        sprintf(buf, "fs%d.charset.name", num);
+        snprintf(buf, sizeof(buf), "fs%d.charset.name", num);
         _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
 
         if( count < 1){
-            sprintf(buf, "fs%d.charset", num);
+            snprintf(buf, sizeof(buf), "fs%d.charset", num);
             _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
             if (count < 1)
                 break;
@@ -2037,8 +1936,8 @@ init_om(
 	data = add_data(om);
 	if (data == NULL)
 	    return False;
-	
-	charset_list = (XlcCharSet *) Xmalloc(sizeof(XlcCharSet) * count);
+
+	charset_list = Xmalloc(sizeof(XlcCharSet) * count);
 	if (charset_list == NULL)
 	    return False;
 	data->charset_list = charset_list;
@@ -2047,12 +1946,12 @@ init_om(
 	while (count-- > 0){
 	    *charset_list++ = _XlcGetCharSet(*value++);
         }
-        sprintf(buf, "fs%d.charset.udc_area", num);
+        snprintf(buf, sizeof(buf), "fs%d.charset.udc_area", num);
         _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
         if( count > 0){
             UDCArea udc;
             int i,flag = 0;
-            udc = (UDCArea)Xmalloc(count * sizeof(UDCAreaRec));
+            udc = Xmalloc(count * sizeof(UDCAreaRec));
 	    if (udc == NULL)
 	        return False;
             for(i=0;i<count;i++){
@@ -2071,10 +1970,10 @@ init_om(
 	    }
         }
 
-        sprintf(buf, "fs%d.font.primary", num);
+        snprintf(buf, sizeof(buf), "fs%d.font.primary", num);
         _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
         if (count < 1){
-            sprintf(buf, "fs%d.font", num);
+            snprintf(buf, sizeof(buf), "fs%d.font", num);
             _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
             if (count < 1)
                 return False;
@@ -2087,7 +1986,7 @@ init_om(
 	data->font_data = font_data;
 	data->font_data_count = count;
 
-        sprintf(buf, "fs%d.font.substitute", num);
+        snprintf(buf, sizeof(buf), "fs%d.font.substitute", num);
         _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
         if (count > 0){
             font_data = read_EncodingInfo(count,value);
@@ -2096,7 +1995,7 @@ init_om(
             data->substitute      = font_data;
             data->substitute_num = count;
         } else {
-            sprintf(buf, "fs%d.font", num);
+            snprintf(buf, sizeof(buf), "fs%d.font", num);
             _XlcGetResource(lcd, "XLC_FONTSET", buf, &value, &count);
             if (count < 1) {
                 data->substitute      = NULL;
@@ -2112,30 +2011,32 @@ init_om(
     }
 
     /* required charset list */
-    required_list = (char **) Xmalloc(sizeof(char *) * gen->data_num);
+    required_list = Xmalloc(sizeof(char *) * gen->data_num);
     if (required_list == NULL)
 	return False;
 
-    bufptr = (char *) Xmalloc(length);
-    if (bufptr == NULL) {
-	Xfree(required_list);
-	return False;
-    }
-
     om->core.required_charset.charset_list = required_list;
     om->core.required_charset.charset_count = gen->data_num;
 
     count = gen->data_num;
     data = gen->data;
 
-    for ( ; count-- > 0; data++) {
-	strcpy(bufptr, data->font_data->name);
-	*required_list++ = bufptr;
-	bufptr += strlen(bufptr) + 1;
+    if (count > 0) {
+	bufptr = Xmalloc(length);
+	if (bufptr == NULL) {
+	    Xfree(required_list);
+	    return False;
+	}
+
+	for ( ; count-- > 0; data++) {
+	    strcpy(bufptr, data->font_data->name);
+	    *required_list++ = bufptr;
+	    bufptr += strlen(bufptr) + 1;
+	}
     }
 
     /* orientation list */
-    orientation = (XOrientation *) Xmalloc(sizeof(XOrientation) * 2);
+    orientation = Xmalloc(sizeof(XOrientation) * 2);
     if (orientation == NULL)
 	return False;
 
@@ -2165,7 +2066,7 @@ _XomGenericOpenOM(XLCd lcd, Display *dpy, XrmDatabase rdb,
     om = create_om(lcd, dpy, rdb, res_name, res_class);
     if (om == NULL)
 	return (XOM) NULL;
-    
+
     if (init_om(om) == False)
 	goto err;
 
